/*-------------------------------------------------------------------------
--
--  config.c - functiion to read the config file for the runtime DNS
--  pattern analysis  system.
--
--  Copyright (c) 1992, 1993, 1994, 1995
--                University of Southern California, Los Angeles, CA
--                All rights reserved.
--
--  Redistribution and use of this software in source or binary form is
--  permitted given that the above copyright notice and this paragraph are
--  duplicated in all such forms and that any documentation and material
--  related to such distribution and use acknowledge that the software
--  was developed by University of Southern California.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMIT TO, THE IMPLIED
--  WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--  THE UNIVERSITY AND THE AUTHORS ASSUME NO RESPONSIBILITY FOR DAMAGES
--  OF ANY KIND RESULTING FROM THE USE OF THIS SOFTWARE.
--
--  Please contact Dr. Peter Danzig of Computer Science Department at
--  danzig@usc.edu for further information.
--
--
===========================================================================
--                           UPDATE HISTORY                              --
--                                                                       --
-- VER     Date         Explanation of Changes              Author       --
---------------------------------------------------------------------------
--  0  | 12/5/92  | Initial version                 | Steve Miller - USC --
--  1  |  4/4/94  | Modified for PEC                | Charles Chan - USC --
--  2  |  6/95    | Fixed formatting                | Steve Miller - USC --
-------------------------------------------------------------------------*/

#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <netdb.h>
#include "checker.h"

FILE           *mfp = NULL;
static char     srvname[200];
static u_long   srvaddr = 0;
static u_long   checker_enabled = 0;

extern char     trace_file_name[100];
extern char     output_file_name[100];
extern char     logging_file_name[100];
extern char     dig_path[100];
extern char     base_password[50];
extern u_long   qr_deletion_time;
extern u_long   negative_cache_time;
extern u_long   nocheck_authority;
extern u_long   echeck_mask;
extern u_long   ck_debug_flag;
extern u_long   translate_address;
extern u_long   password_check;
extern u_long   disable_tq_flag;
extern u_long   error_threshold;
extern u_long   memory_logging;
extern u_long   packet_tracing;
extern u_long   local_domain_checking;
extern u_long   port_53_checking;
extern u_long   maintenance_interval;

extern FILE *ofp;
extern FILE *tfp;


/*-----------------------------------------------------------------------
--
--  A routine to read the configuration file and set the variables
--  as indicated.
--
-----------------------------------------------------------------------*/
int 
read_configuration()
    {
    static init = 0;
    FILE *fp, *lfp;
    char line[100];
    char *t1, *t2, t3[100];
    u_long tmp;
    char   tmp1[100];
    static char eqstring[40] = "======================================";
    struct timeval  tm;
    char            tstr[100];
  
    /*
     * Open the log file, use a default name if an earlier configuration file
     * has not set a new one.
     */

    lfp = open_logging_file(logging_file_name);
    if(lfp) fprintf(lfp, "%s%s\n", eqstring, eqstring);
    gettimeofday(&tm, (struct timezone *) 0);
    strcpy(tstr, ctime(&tm.tv_sec));
    tstr[24] = '\0';

    /*
     * Go find the configuration file.  Look first in our local
     * directory, and if we can't find one there then go look for a
     * default configuration in the /etc directory. 
     */

    if ((fp = fopen("./checker.cfg", "r")) == NULL) {

        if ((fp = fopen("/etc/checker.cfg", "r")) == NULL) {

             if( ! init ) {
                 fprintf(stderr, "cannot open checker.cfg, exiting.\n");
                 exit (0); }
             else {
                 if(lfp) fprintf(lfp, 
                     "%s| Checker-err cannot open checker.cfg.\n", tstr);
                 if(lfp) fprintf(lfp, "%s%s\n", eqstring, eqstring);
                 fclose(lfp);
                 return; }
             }

    /*
     * If we were able to open the configuration file then log which one we
     * got to the log file.
     */

        else if(lfp) fprintf(lfp, 
            "%s| Checker-err reading config file /etc/checker.cfg\n", tstr ); }
    else if(lfp) fprintf(lfp, 
        "%s| Checker-err reading config file ./checker.cfg\n", tstr );

    /* Get a line and get the third token as a parameter. */

    while (fgets(line, 100, fp) != NULL) {

        if (line[0] == '#') continue;
        sscanf(line, "%*s %*s %s", t3);

        if (strncmp(line, "debug flag", 10) == 0) {
            sscanf(t3, "%d", &ck_debug_flag);
            if (lfp) fprintf(lfp, "set debug flag to %d\n", ck_debug_flag);
            }

        else if (strncmp(line, "error mask", 10) == 0) {
            sscanf(t3, "%X", &echeck_mask);
            if (lfp) fprintf(lfp, "set echeck mask to %X\n", echeck_mask);
            }

        else if (strncmp(line, "password check", 14) == 0) {
            if (strncmp("off", t3, 3) == 0) password_check = 0;
            else                            password_check = 1;
            if (lfp) fprintf(lfp, "set password check flag to %d\n",
               password_check);
            }

        else if (strncmp(line, "password string", 15) == 0) {
            sscanf(t3, "%s", base_password);
            if (lfp) fprintf(lfp, "encrypted password string is now %s\n",
                base_password);
            }

        else if (strncmp(line, "time queue", 10) == 0) {
            if (strncmp("off", t3, 3) == 0) disable_tq_flag = 1;
            else                            disable_tq_flag = 0;
            if (lfp) fprintf(lfp, "set disable tq flag to %d\n",
               disable_tq_flag);
            }

        else if (strncmp(line, "domain checking", 15) == 0) {
            if (strncmp("off", t3, 3) == 0) local_domain_checking = 0;
            else                            local_domain_checking = 1;
            if (lfp) fprintf(lfp, "domain checking flag is now %d.\n",
               local_domain_checking);
            }

        else if (strncmp(line, "port53 checking", 15) == 0) {
            if (strncmp("off", t3, 3) == 0) port_53_checking = 0;
            else                            port_53_checking = 1;
            if (lfp) fprintf(lfp, "port 53 checking flag is now %d.\n",
               port_53_checking);
            }

        else if (strncmp(line, "checker enabled", 15) == 0) {
            if (strncmp("off", t3, 3) == 0) checker_enabled = 0;
            else                            checker_enabled = 1;
            if (lfp) fprintf(lfp, "checker enable flag is now %d.\n",
               checker_enabled);
            }

        else if (strncmp(line, "error threshold", 15) == 0) {
            sscanf(t3, "%d", &error_threshold);
            if (lfp) fprintf(lfp, "error threshold is %d\n", error_threshold);
            }

        else if (strncmp(line, "deletion time", 13) == 0) {
            sscanf(t3, "%d", &qr_deletion_time);
            if (lfp) fprintf(lfp, "qr deletion time is %d\n",
               qr_deletion_time);
            }

        else if (strncmp(line, "negative cache", 14) == 0) {
            sscanf(t3, "%d", &negative_cache_time);
            if (lfp) fprintf(lfp, "negative_cache_time is %d\n",
               negative_cache_time);
            }

        else if (strncmp(line, "authority check", 15) == 0) {
            if (strncmp("off", t3, 3) == 0) nocheck_authority = 1;
            else                            nocheck_authority = 0;
            if (lfp) fprintf(lfp, "nocheck_authority = %d\n",
               nocheck_authority);
            }

        else if (strncmp(line, "packet trace", 12) == 0) {
            if (strncmp("off", t3, 3) == 0) packet_tracing = 0;
            else                            packet_tracing = 1;
            if (lfp) fprintf(lfp, "packet_tracing = %d\n", packet_tracing );
            }

        else if (strncmp(line, "memory logging", 14) == 0) {
            if (strncmp("off", t3, 3) == 0) memory_logging = 0;
            else                            memory_logging = 1;
            if (lfp) fprintf(lfp, "memory_logging = %d\n", memory_logging);
            }

        else if (strncmp(line, "translate address", 17) == 0) {
            if (strncmp("off", t3, 3) == 0) translate_address = 0;
            else                            translate_address = 1;
            if (lfp) fprintf(lfp, "translate_address = %d\n",
               translate_address);
            }

        else if (strncmp(line, "maintenance interval", 20) == 0) {
            sscanf(t3, "%d", &maintenance_interval);
            if (lfp) fprintf(lfp, "maintenance_interval = %d\n",
               maintenance_interval);
            }

        else if (strncmp(line, "memory debug-file", 17) == 0) {
            if (mfp) { fclose(mfp); mfp = NULL; }
            if (strcmp(t3, "off") != 0) {
                if ((mfp = fopen(t3, "a")) == NULL) {
                    if(lfp) fprintf(lfp, "unable to open memory trace.\n" ); }
                if (lfp) fprintf(lfp, "set memory trace file to %s\n", t3);
                }
            }

        else if (strncmp(line, "logging file", 12) == 0) {
            sscanf(t3, "%s", tmp1);
            if (strcmp(tmp1, logging_file_name) != 0) {
                strcpy(logging_file_name, tmp1);
                if (lfp) {
                    fprintf(lfp, "New logging file name is %s\n",
                        logging_file_name);
                    fprintf(lfp, "%s%s\n", eqstring, eqstring);
                    fclose(lfp); }
                lfp = open_logging_file(logging_file_name);
                fprintf(lfp, "%s%s\n", eqstring, eqstring);
                }
            else if (lfp)
                fprintf(lfp, "logging file name is %s\n",
                    logging_file_name);
            }

        else if (strncmp(line, "output file", 11) == 0) {
            sscanf(t3, "%s", output_file_name);
            if (lfp) fprintf(lfp, "output reports file name is now %s\n",
               output_file_name);
            }

        else if (strncmp(line, "trace file", 10) == 0) {
            sscanf(t3, "%s", tmp1);
            set_trace_nam(tmp1);
            if (lfp) fprintf(lfp, "binary packet trace file name is now %s\n",
               trace_file_name);
            }

        else if (strncmp(line, "dig path", 8) == 0) {
            sscanf(t3, "%s", dig_path);
            if (lfp) fprintf(lfp, "dig path name is now %s\n",
               dig_path);
            }
        }

    /* close the logging and configuration files */

    if (lfp) { fprintf(lfp, "%s%s\n", eqstring, eqstring); fclose(lfp); }
    fclose(fp);
    init = 1;
    }


/*-----------------------------------------------------------------------
--
--  A simple routine to open a logging file.
--
-----------------------------------------------------------------------*/
FILE *open_logging_file(logging_file_name)
char *logging_file_name;
    {
    FILE           *lfp;
    struct timeval  tm;

    /*
     * Open the log file, use a default name if an earlier configuration file
     * has not opened one already.
     */

    if ((lfp = fopen(logging_file_name, "a")) == NULL) {
        fprintf(stderr, "cannot open %s as checker's log file\n",
            logging_file_name);
        }

    return (lfp);
    }


/*-----------------------------------------------------------------------
--
--  A simple routine to write a logging file text entry.
--
-----------------------------------------------------------------------*/
void 
write_log_entry(text)
char *text;
    {
    FILE           *lfp;
    struct timeval  tm;
    char            tstr[100];

    /* Open the log file. */

    if ((lfp = fopen(logging_file_name, "a")) == NULL) {
        fprintf(stderr, "cannot open %s as checker's log file\n",
            logging_file_name);
        }

    /* Write the current time and the desired text into the log file. */

    if (lfp) {
        gettimeofday(&tm, (struct timezone *) 0);
        strcpy(tstr, ctime(&tm.tv_sec));
        tstr[24] = '\0';
        fprintf(lfp, "%s| %s\n", tstr, text);
        fclose(lfp);
        }
    }


/*-----------------------------------------------------------------------
--
--  A simple routine to dump out the current state of the malloc
--  buffer space taken from the heap.
--
-----------------------------------------------------------------------*/
dump_mallinfo()
    {
    struct mallinfo mi;
    struct timeval  tm;
    FILE           *lfp;
    char            lstr[100];

    /* Get the malloc database information structure. */

    mi = mallinfo();

    /* Format an output string and write it to the log file. */

    sprintf(lstr, "uordblks = %d", mi.uordblks);
    write_log_entry(lstr);

    /*
     * WARNING: Some items here are SunOS specific. 
     *
     *     printf( "arena     = %d\n", mi.arena ); 
     *     printf( "ordblks   = %d\n", mi.ordblks ); 
     *     printf( "smblks    = %d\n", mi.smblks ); 
     *     printf( "hblks     = %d\n", mi.hblks ); 
     *     printf( "hblkhd    = %d\n", mi.hblkhd ); 
     *     printf( "usmblks   = %d\n", mi.usmblks ); 
     *     printf( "fsmblks   = %d\n", mi.fsmblks ); 
     *     printf( "uordblks  = %d\n", mi.uordblks ); 
     *     printf( "fordblks  = %d\n", mi.fordblks ); 
     *     printf( "keepcost  = %d\n", mi.keepcost ); 
     *     printf( "mxfast    = %d\n", mi.mxfast ); 
     *     printf( "nlblks    = %d\n", mi.nlblks ); 
     *     printf( "grain     = %d\n", mi.grain ); 
     *     printf( "uordbytes = %d\n", mi.uordbytes ); 
     *     printf( "allocated = %d\n", mi.allocated ); 
     *     printf( "treeoverh = %d\n", mi.treeoverhead );
     */

    }


/*------------------------------------------------------------------------
--
--  set_trace_nam() - a simple routine to save and open the binary
--  packet trace file.
--
--  parameters:
--      oname - output file name
--
--  globals:
--      sets the trace file pointer.
--
--  return value:
--      none
--
--  notes:
--      none
--
------------------------------------------------------------------------*/
void 
set_trace_nam(oname)
char *oname;
    {

    /* If there is an existing file then close it */

    if (tfp != NULL) { fclose(tfp); tfp = NULL; }

    /* Save the name */

    strcpy(trace_file_name, oname);

    /* Look to see if the user has decided to turn off the trace file */

    if( strcmp( trace_file_name, "off"  ) == 0 ||
        strcmp( trace_file_name, "none" ) == 0 ) return;

    /* Try to open it. */

    if ((tfp = fopen(trace_file_name, "a")) == NULL) {
        fprintf(stderr, "could not open %s as packet trace file\n",
            trace_file_name);
        return;
        }
    }
